{-# LANGUAGE DuplicateRecordFields #-}
{-# LANGUAGE NamedFieldPuns        #-}

module Schpin.Walker.Pkg where

import           Control.Monad.Reader           ( runReader )
import           Schpin.Launch
import           Schpin.Launch.Xml              ( writeLaunchXML )
import           Schpin.Package                 ( PackageConfig
                                                , pkg_dir
                                                , pkg_info
                                                , pkg_name
                                                , srdf_subdir
                                                , urdf_subdir
                                                )
import           Schpin.SCAD                    ( renderSCAD )
import           Schpin.SRDF.Xml                ( writeSRDFXML )
import           Schpin.URDF.Xml                ( writeURDFXML )
import           Schpin.Walker                  ( SimpleWalker
                                                , extendWalker
                                                , walker_name
                                                )
import           Schpin.Walker.SCAD             ( getSCADFiles
                                                , renderWalkerStl
                                                )
import           Schpin.Walker.SRDF             ( getSRDF )
import           Schpin.Walker.URDF             ( getURDF )


writeRobotDescriptionLaunch :: SimpleWalker -> PackageConfig -> IO ()
writeRobotDescriptionLaunch walker cfg = writeLaunchXML
  def
  (pkg_dir cfg ++ "/launch/robot_description.launch")
 where
  pname = pkg_name (pkg_info cfg)
  def   = (defLaunch :: Launch)
    { params = [ (defParam :: Param)
                 { name     = "robot_description"
                 , textfile = Just
                              $  "$(find "
                              ++ pname
                              ++ ")/src/"
                              ++ walker_name walker
                              ++ ".urdf"
                 }
               , (defParam :: Param)
                 { name     = "robot_semantics"
                 , textfile = Just
                              $  "$(find "
                              ++ pname
                              ++ ")/src/"
                              ++ walker_name walker
                              ++ ".srdf"
                 }
               ]
    }

writeCMakeLists :: PackageConfig -> IO ()
writeCMakeLists cfg = writeFile (pkg_dir cfg ++ "/CMakeLists.txt") content
 where
  content :: String
  content = unlines
    [ "cmake_minimum_required(VERSION 3.5)"
    , "project(" ++ pkg_name (pkg_info cfg) ++ ")"
    , "find_package(ament_cmake REQUIRED)"
    , "ament_package()"
    ]


generateWalkerPackage :: SimpleWalker -> PackageConfig -> IO ()
generateWalkerPackage walker pc = do
  let extended_walker = extendWalker walker pc
  mapM_ (uncurry renderSCAD) $ getSCADFiles extended_walker
  writeURDFXML
    (getURDF extended_walker)
    (  pkg_dir pc
    ++ "/"
    ++ urdf_subdir pc
    ++ "/"
    ++ walker_name extended_walker
    ++ ".urdf"
    )
  writeSRDFXML
    (getSRDF extended_walker)
    (  pkg_dir pc
    ++ "/"
    ++ srdf_subdir pc
    ++ "/"
    ++ walker_name extended_walker
    ++ ".srdf"
    )
  renderWalkerStl extended_walker
